// 3dfx Programming Secrets - gourauddemo
// Implementing gouraud shading have NEVER been simplier!

// This program is a WIN32 consol program that draws a gouradshaded
// triangle and changes
// the colors as the user presses a key until the user presses
// the escape key.
// This program was built using the LccWin32 compiler/linker.

#include <glide.h>

// our own triangle type:
typedef struct Triangle
{
	GrVertex v1;
	GrVertex v2;
	GrVertex v3;
} Triangle_t;

// a array type holding 10 different triangles:
typedef Triangle_t Triangles_t[10];

void main(void)
{
  int loop;
  char key = 0;
  // our triangles:
  Triangles_t GourTris;

  GrHwConfiguration HwConfig;

  // Initialise Glide - this function MUST be called
  // before anything else is done with Glide:
  grGlideInit();

  // Check for precence of 3dfx hardware:
  if ((grSstQueryHardware(&HwConfig)) == FXFALSE)
  {
        printf("Error! No 3dfx hardware!");
		exit(1);
  }
  // found 3dfx hardware!
  // select first voodoo board in system - this means you
  // can use different boards in your computer connected to
  // different monitors from the same program! Glide
  // only works with the current board selected from
  // the grSstSelect function.
  grSstSelect( 0 );

  // sets 3dfx videomode with some parameters
  // It's kinda selfexplained, check out the GLIDE docs if
  // you want to see all possible combinations!
  // Please note that this demo just works with fullscreen only 3dfx
  // cards like the voodoo and voodoo2.
  grSstWinOpen( 0,
               GR_RESOLUTION_640x480,
               GR_REFRESH_60Hz,
               GR_COLORFORMAT_RGBA,
               GR_ORIGIN_UPPER_LEFT,
               2, 1);

  // clears the backbuffer black:
  grBufferClear( 0x00000000, 0, GR_WDEPTHVALUE_FARTHEST );

  // swaps the backbuffer to the frontbuffer making a
  // black screen visible:
  grBufferSwap(1);

  // setup the 3dfx card to produce Gouraud shaded triangles:
  grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
  	GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_NONE, FXFALSE);

  // Setup the triangles:
  for (loop=0; loop<10; loop++)
  {
    GourTris[loop].v1.x = 200.0;
	GourTris[loop].v1.y = 200.0;
	GourTris[loop].v2.x = 400.0;
	GourTris[loop].v2.y = 200.0;
	GourTris[loop].v3.x = 300.0;
	GourTris[loop].v3.y = 400.0;
    GourTris[loop].v1.r = (float)(255-loop*25);
	GourTris[loop].v1.g = (float)(loop*25);
    GourTris[loop].v1.b = (float)(loop*25);
    GourTris[loop].v2.r = (float)(loop*25);
	GourTris[loop].v2.g = (float)(255-loop*25);
    GourTris[loop].v2.b = (float)(255-loop*25);
    GourTris[loop].v3.r = (float)(100-loop*10);
	GourTris[loop].v3.g = (float)(loop*10);
    GourTris[loop].v3.b = (float)(loop*10);
  }

  // Here we renders the triangle:
  loop = 0;
  while (key != 27)
  {
          // clears the backbuffer black:
          grBufferClear( 0x00000000, 0, GR_WDEPTHVALUE_FARTHEST );

	  if (loop == 10) loop = 0;

	  grDrawTriangle(&GourTris[loop].v1,&GourTris[loop].v2,
	  	&GourTris[loop].v3);
	  grBufferSwap(1);
	  loop++;
	  key = getch();
  }



  // shut down glide and return to normal videomode:
  grGlideShutdown();

}

